@@ -139,6 +139,11 @@ static PyObject* vec_new(PyTypeObject* type, PyObject* args, PyObject* kw) {
 	return (PyObject*)self;
 }
 
+static PyObject* vec_reduce(vec_obj* obj)
+{
+	return Py_BuildValue("O(ffff)", Py_TYPE(obj), obj->v[0], obj->v[1], obj->v[2], obj->v[3]);
+}
+
 static PyObject* vec_str(vec_obj * self)
 {
 	char buf[256];
@@ -170,14 +175,11 @@ static PyObject* vec_richcompare(PyObject * self, PyObject * other, int op)
 		for (int i = 0; i < d1; i++) {
 			if (!vmath_almostEqual(v1[i], v2[i])) {
 				eq = false;
-				break;
 			}
 		}
 	}
-	PyObject* result;
-	if (op == Py_NE) eq ^= eq;
-	if (op == Py_EQ) result = Py_True;
-	else  result = Py_False;
+	if (op == Py_NE) eq = !eq;
+	PyObject* result = eq?Py_True:Py_False;
 	Py_INCREF(result);
 	return result;
 }
@@ -1136,6 +1138,31 @@ static PyObject* quat_rotation(PyObject * self, PyObject * args)
 	return (PyObject*)obj;
 }
 
+static PyObject* quat_look_rotation(PyObject * self, PyObject * args)
+{
+	PyObject* a;
+	PyObject* b = nullptr;
+	if (!PyArg_ParseTuple(args, "OO", &a, &b)) return NULL;
+
+	int d1 = 3;
+	int	d2 = 3;
+	float buff1[4];
+	float buff2[4] = {0, 0, 1, 0};
+	float* v1 = pyObjToFloat((PyObject*)a, buff1, d1);
+	if (!v1 || d1 < 3) return NULL;
+
+	float* v2 = buff2;
+	if(b) v2 = pyObjToFloat((PyObject*)b, buff2, d2);
+	if (!v2 || d2 < 3) return NULL;
+
+	vec_obj* obj = PyObject_New(vec_obj, &QuatType);
+	if (!obj) return NULL;
+
+	vmath_quat_look_rotation(v1, v2, obj->v);
+	obj->d = 4;
+	return (PyObject*)obj;
+}
+
 static PyObject* quat_rotationX(PyObject * self, PyObject * args)
 {
 	float r;
@@ -1326,6 +1353,19 @@ static PyObject* mat_new(PyTypeObject * type, PyObject * args, PyObject * kw) {
 	return (PyObject*)self;
 }
 
+static PyObject* mat_reduce(mat_obj* obj)
+{
+	switch (obj->d) {
+	case 2:
+		return Py_BuildValue("O(ffff)", Py_TYPE(obj), obj->m[0], obj->m[1], obj->m[2], obj->m[3]);
+	case 3:
+		return Py_BuildValue("O(fffffffff)", Py_TYPE(obj), obj->m[0], obj->m[1], obj->m[2], obj->m[3], obj->m[4], obj->m[5], obj->m[6], obj->m[7], obj->m[8]);
+	case 4:
+		return Py_BuildValue("O(ffffffffffffffff)", Py_TYPE(obj), obj->m[0], obj->m[1], obj->m[2], obj->m[3], obj->m[4], obj->m[5], obj->m[6], obj->m[7], obj->m[8], obj->m[9], obj->m[10], obj->m[11], obj->m[12], obj->m[13], obj->m[14], obj->m[15]);
+	}
+}
+
+
 static PyObject* mat_str(mat_obj * self)
 {
 	char buf[512];
@@ -1623,21 +1663,18 @@ static PyObject* mat_inverse(PyObject * self, PyObject * args) {
 		return  NULL;
 	}
 
-	PyObject* obj;
-
 	if (a->ob_type == &QuatType) {
-		obj = (PyObject*)PyObject_New(vec_obj, a->ob_type);
-		vmath_quat_inverse(((vec_obj*)a)->v, ((vec_obj*)obj)->v);
-		((vec_obj*)obj)->d = 4;
+		vec_obj* obj = PyObject_New(vec_obj, a->ob_type);
+		vmath_quat_inverse(((vec_obj*)a)->v, obj->v);
+		obj->d = 4;
+		return (PyObject*)obj;
 	}
 	else {
-		obj = (PyObject*)PyObject_New(mat_obj, a->ob_type);
-		vmath_mat_inverse(((mat_obj*)a)->m, ((mat_obj*)a)->d, ((mat_obj*)obj)->m);
-		((mat_obj*)obj)->d = ((mat_obj*)a)->d;
+		mat_obj* obj = PyObject_New(mat_obj, a->ob_type);
+		vmath_mat_inverse(((mat_obj*)a)->m, ((mat_obj*)a)->d, obj->m);
+		obj->d = ((mat_obj*)a)->d;
+		return (PyObject*)obj;
 	}
-
-	return (PyObject*)obj;
-
 }
 
 static PyObject* mat_orthoInverse(PyObject * self, PyObject * args) {
@@ -1765,6 +1802,7 @@ static PyObject* mat_transform(PyObject* self, PyObject* args) {
 		PyErr_SetString(PyExc_ValueError, "invalid arguments");
 		return  NULL;
 	}
+
 	float* shear = shear_o ? ((mat_obj*)shear_o)->m : nullptr;
 
 	mat_obj* obj = PyObject_New(mat_obj, &Mat44Type);
@@ -1838,8 +1876,8 @@ static PyObject* mat_lookAt(PyObject * self, PyObject * args) {
 static PyObject* mat_perspective(PyObject * self, PyObject * args, PyObject* kwdargs) {
 	float fovyRadians, aspect, zNear, zFar;
 
-	static char* keywords[]  = { "fovyRadians", "aspect", "zNear", "zFar", NULL };
-	if (!PyArg_ParseTupleAndKeywords(args, kwdargs, "ffff", keywords, &fovyRadians, &aspect, &zNear, &zFar)) return NULL;
+	static const char* keywords[] = { "fovyRadians", "aspect", "zNear", "zFar", NULL };
+	if (!PyArg_ParseTupleAndKeywords(args, kwdargs, "ffff", (char**)keywords, &fovyRadians, &aspect, &zNear, &zFar)) return NULL;
 
 	mat_obj* obj = PyObject_New(mat_obj, &Mat44Type);
 	if (!obj) return NULL;
@@ -2141,6 +2179,12 @@ static PyObject* aabb_new(PyTypeObject* type, PyObject* args, PyObject* kw) {
 	return (PyObject*)self;
 }
 
+static PyObject* aabb_reduce(aabb_obj* obj)
+{
+	return Py_BuildValue("O(ffffff)", Py_TYPE(obj), obj->min[0], obj->min[1], obj->min[2], obj->max[0], obj->max[1], obj->max[2]);
+}
+
+
 static PyObject* aabb_str(aabb_obj* self)
 {
 	char buf[256];
@@ -2260,6 +2304,11 @@ static PyObject* euler_new(PyTypeObject* type, PyObject* args, PyObject* kw) {
 	return (PyObject*)self;
 }
 
+static PyObject* euler_reduce(vec_obj* obj)
+{
+	return Py_BuildValue("O(ffff)", Py_TYPE(obj), obj->v[0], obj->v[1], obj->v[2], obj->v[3]);
+}
+
 static PyObject* euler_toQuat(vec_obj* self) 
 {
 	vec_obj* obj = PyObject_New(vec_obj, &QuatType);
@@ -2345,7 +2394,8 @@ static PyMethodDef vec_methods[] = {
 	{ "setElem",	(PyCFunction)vec_setElem,	METH_VARARGS, vec_setElem_doc },
 	{ "getElem",	(PyCFunction)vec_getElem,	METH_VARARGS, vec_getElem_doc },
 	{ "normalize",	(PyCFunction)vec_self_normalize,METH_VARARGS,normalize_doc },
-	{ NULL,	NULL }
+	{"__reduce__",  (PyCFunction)vec_reduce,    METH_NOARGS, NULL},
+{ NULL,	NULL }
 };
 
 static PyMethodDef mat_methods[] = {
@@ -2355,6 +2405,7 @@ static PyMethodDef mat_methods[] = {
 	{ "getRow",		(PyCFunction)mat_getRow,	METH_VARARGS, mat_getRow_doc },
 	{ "setCol",		(PyCFunction)mat_setCol,	METH_VARARGS, mat_setCol_doc },
 	{ "setRow",		(PyCFunction)mat_setRow,	METH_VARARGS, mat_setRow_doc },
+	{"__reduce__",  (PyCFunction)mat_reduce,    METH_NOARGS, NULL},
 	{ NULL,	NULL }
 };
 
@@ -2366,6 +2417,7 @@ static PyMethodDef mat4_methods[] = {
 	{ "setCol",		(PyCFunction)mat_setCol,	METH_VARARGS, mat_setCol_doc },
 	{ "setRow",		(PyCFunction)mat_setRow,	METH_VARARGS, mat_setRow_doc },
 	{ "getTransform",(PyCFunction)mat_getTransform,	METH_VARARGS, mat_getTransform_doc },
+	{"__reduce__",  (PyCFunction)mat_reduce,    METH_NOARGS, NULL},
 { NULL,	NULL }
 };
 
@@ -2374,6 +2426,7 @@ static PyMethodDef euler_methods[] = {
 	{ "setElem",	(PyCFunction)vec_setElem,	METH_VARARGS, vec_setElem_doc },
 	{ "getElem",	(PyCFunction)vec_getElem,	METH_VARARGS, vec_getElem_doc },
 	{ "toQuat",		(PyCFunction)euler_toQuat,	METH_NOARGS, toQuat_doc },
+	{"__reduce__",  (PyCFunction)euler_reduce,    METH_NOARGS, NULL},
 	{ NULL,	NULL }
 };
 
@@ -2504,7 +2557,9 @@ static PyMethodDef aabb_methods[] = {
 	{"insert", (PyCFunction)aabb_insert, METH_VARARGS, insert_doc},
 	{"repair", (PyCFunction)aabb_repair, METH_VARARGS, repair_doc},
 	{"isInside", (PyCFunction)aabb_isInside, METH_VARARGS, isInside_doc},
-	{ nullptr, nullptr, 0, nullptr }
+	{"__reduce__",  (PyCFunction)aabb_reduce,    METH_NOARGS, NULL},
+
+{ nullptr, nullptr, 0, nullptr }
 };
 
 PyTypeObject Vec2Type = {
@@ -2900,6 +2955,7 @@ static PyMethodDef vmath_methods[] = {
 	{ "lerp", (PyCFunction)vec_lerp, METH_VARARGS, lerp_doc},
 	{ "slerp", (PyCFunction)vec_slerp, METH_VARARGS, slerp_doc},
 	{"quat_rotation", (PyCFunction)quat_rotation, METH_VARARGS, quat_rotation_doc},
+	{"quat_look_rotation", (PyCFunction)quat_look_rotation, METH_VARARGS, quat_look_rotation_doc},
 	{"quat_rotationX", (PyCFunction)quat_rotationX, METH_VARARGS, quat_rotationX_doc},
 	{"quat_rotationY", (PyCFunction)quat_rotationY, METH_VARARGS, quat_rotationY_doc},
 	{"quat_rotationZ", (PyCFunction)quat_rotationZ, METH_VARARGS, quat_rotationZ_doc},
