@@ -2,18 +2,22 @@
 #include <float.h>
 #include <math.h>
 
+#ifndef PI
+    #define PI 3.14159f
+#endif
+
 inline void vmath_cpy(const float* v, int d, float* out) {
 	for (int i = 0; i < d; i++) out[i] = v[i];
 }
 
-inline bool vmath_almostEqual(float a, float b) {
-	return fabs(a - b) < FLT_EPSILON;
+inline bool vmath_almostEqual(float a, float b, float margin = 0.00001f) {
+	return fabs(a - b) < margin;
 }
 
-inline bool vmath_almostEqual(const float* v0, const float* v1, int d) {
+inline bool vmath_almostEqual(const float* v0, const float* v1, int d, float margin = 0.00001f) {
 
 	for (int i = 0; i < d; i++) {
-		if (!vmath_almostEqual(v0[i], v1[i])) return false;
+		if (!vmath_almostEqual(v0[i], v1[i], margin)) return false;
 	}
 	return true;
 }
@@ -268,6 +272,59 @@ inline void vmath_quat_rotationZ(float radians, float* out)
 	out[3] = c;
 }
 
+// [IGE] based on http://answers.unity3d.com/questions/467614/what-is-the-source-code-of-quaternionlookrotation.html
+inline void vmath_quat_look_rotation(const float* forward, const float* up, float* quaternion)
+{
+	float vector[3], vector2[3], vector3[3];
+	vmath_normalize(forward, 3, vector);
+	vmath_cross3D(up, vector, vector2);
+	vmath_normalize(vector2, 3, vector2);
+	vmath_cross3D(vector, vector2, vector3);
+
+	float m00 = vector2[0], m01 = vector2[1], m02 = vector2[2];
+	float m10 = vector3[0], m11 = vector3[1], m12 = vector3[2];
+	float m20 = vector[0], m21 = vector[1], m22 = vector[2];
+
+	float num8 = (m00 + m11) + m22;
+	if (num8 > 0.f)
+	{
+		float num = (float)sqrt(1.0 + num8);
+		quaternion[3] = num * 0.5f;
+		num = 0.5f / num;
+		quaternion[0] = (m12 - m21) * num;
+		quaternion[1] = (m20 - m02) * num;
+		quaternion[2] = (m01 - m10) * num;
+		return;
+	}
+	if ((m00 >= m11) && (m00 >= m22))
+	{
+		float num7 = (float)sqrt(((1.0 + m00) - m11) - m22);
+		float num4 = 0.5f / num7;
+		quaternion[0] = 0.5f * num7;
+		quaternion[1] = (m01 + m10) * num4;
+		quaternion[2] = (m02 + m20) * num4;
+		quaternion[3] = (m12 - m21) * num4;
+		return;
+	}
+	if (m11 > m22)
+	{
+		float num6 = (float)sqrt(((1.0 + m11) - m00) - m22);
+		float num3 = 0.5f / num6;
+		quaternion[0] = (m10 + m01) * num3;
+		quaternion[1] = 0.5 * num6;
+		quaternion[2] = (m21 + m12) * num3;
+		quaternion[3] = (m20 - m02) * num3;
+		return;
+	}
+	float num5 = (float)sqrt(((1.0 + m22) - m00) - m11);
+	float num2 = 0.5f / num5;
+	quaternion[0] = (m20 + m02) * num2;
+	quaternion[1] = (m21 + m12) * num2;
+	quaternion[2] = 0.5 * num5;
+	quaternion[3] = (m01 - m10) * num2;
+	return;
+}
+
 // Use a unit-length quaternion to rotate a 3-D vector
 inline void vmath_quat_rotate(const float* quat, const float* v0, float* out)
 {
@@ -293,13 +350,14 @@ inline void vmath_quat_inverse(const float* quat, float* out)
 {
 	float len = 0.0f;
 	for (int i = 0; i < 4; i++) len += quat[i] * quat[i];
-	out[0] = -quat[0] / len;
-	out[1] = -quat[1] / len;
-	out[2] = -quat[2] / len;
-	out[3] = quat[3] / len;
+	out[0] = -quat[0]/len;
+	out[1] = -quat[1]/len;
+	out[2] = -quat[2]/len;
+	out[3] = quat[3]/len;
 }
 
 
+
 inline void vmath_quat_slerp(float t, const float* quat0, const float* quat1, float* out)
 {
 	float start[4];
@@ -837,6 +895,22 @@ inline void vmath_mat4_orthographic(float left, float right, float bottom, float
 	vmath_vect_set((-sum_rl * inv_rl), (-sum_tb * inv_tb), (sum_nf * inv_nf), 1.0f, 4, &(out[12]));
 }
 
+inline void vmath_mat4_orthographic_ZO(float left, float right, float bottom, float top, float zNear, float zFar, float* out)
+{
+	float sum_rl, sum_tb, sum_nf, inv_rl, inv_tb, inv_nf;
+	sum_rl = (right + left);
+	sum_tb = (top + bottom);
+	sum_nf = (zNear + zFar);
+	inv_rl = (1.0f / (right - left));
+	inv_tb = (1.0f / (top - bottom));
+	inv_nf = (1.0f / (zNear - zFar));
+
+	vmath_vect_set((inv_rl + inv_rl), 0.0f, 0.0f, 0.0f, 4, &(out[0]));
+	vmath_vect_set(0.0f, (inv_tb + inv_tb), 0.0f, 0.0f, 4, &(out[4]));
+	vmath_vect_set(0.0f, 0.0f, (inv_nf), 0.0f, 4, &(out[8]));
+	vmath_vect_set((-sum_rl * inv_rl), (-sum_tb * inv_tb), (zNear * inv_nf), 1.0f, 4, &(out[12]));
+}
+
 
 inline void vmath_quat(const float* mat3, float* out)
 {
@@ -931,25 +1005,46 @@ inline void vmath_eulerToQuat(const float* euler, float* quat) {
 }
 
 inline void vmath_quatToEuler(const float* quat, float* euler) {
-	float test = quat[0] * quat[1] + quat[2] * quat[3];
-	if (test > 0.49999) {
-		euler[0] = 0.0f;
-		euler[1] = 2.0f * atan2f(quat[0], quat[4]);
-		euler[2] = 3.14159f / 2.0f;
-		return;
-	}
-	else if(test < -0.49999f) {
-		euler[0] = 0;
-		euler[1] = -2.0f * atan2f(quat[0], quat[4]);
-		euler[2] = -3.14159f / 2.0f;
+	// ref: https://en.wikipedia.org/wiki/Conversion_between_quaternions_and_Euler_angles
+	// special cases: (0.5f, 0.5f, 0.5f, -0.5f) and (0.5f, 0.5f, -0.5f, 0.5f)
+	if (vmath_almostEqual(quat[0], 0.5f) && vmath_almostEqual(quat[1], 0.5f)) {
+		if (vmath_almostEqual(quat[2], 0.5f) && vmath_almostEqual(quat[3], -0.5f)) {
+			euler[0] = 1.5708f;
+			euler[1] = 1.5708f;
+		}
+		else if (vmath_almostEqual(quat[2], -0.5f) && vmath_almostEqual(quat[3], 0.5f)) {
+			euler[0] = -1.5708f;
+			euler[1] = -1.5708f;
+		}
+		euler[2] = 0.0f;
 		return;
 	}
+
+	// singularity cases
 	float sqx = quat[0]*quat[0];
 	float sqy = quat[1]*quat[1];
 	float sqz = quat[2]*quat[2];
+	float sqw = quat[3]*quat[3];
+	float unit = sqx + sqy + sqz + sqw;
+	float test = quat[0] * quat[1] + quat[2] * quat[3];
+	if (test > 0.49999f * unit) {
+		euler[0] = 0.0f;
+		euler[1] = 2.0f * atan2f(quat[0], quat[3]);
+		euler[2] = PI / 2.0f;
+		return;
+	}
+	else if(test < -0.49999f * unit) {
+		euler[0] = 0.f;
+		euler[1] = -2.0f * atan2f(quat[0], quat[3]);
+		euler[2] = -PI / 2.0f;
+		return;
+	}
+
+	// normal cases
 	euler[0] = atan2f(2.0f * quat[0] * quat[3] - 2.0f * quat[1] * quat[2], 1.0f - 2.0f * sqx - 2.0f * sqz);
 	euler[1] = atan2f(2.0f * quat[1] * quat[3] - 2.0f * quat[0] * quat[2], 1.0f - 2.0f * sqy - 2.0f * sqz);
-	euler[2] = asinf(2.0f * test);
+	euler[2] = vmath_almostEqual(unit, 0.f) ? 0.f : asinf(2.0f * test / unit);
+	return;
 }
 
 //0:tilt, 1:pan, 2:roll
@@ -1000,7 +1095,7 @@ inline void vmath_mat4_fromTransform(const float* pos, const float* rot, int rot
 	rottransmat[13] = pos[1];
 	rottransmat[14] = pos[2];
 	rottransmat[15] = 1.0f;
-	vmath_mul_matrix_matrix(rottransmat, shear, 4, 4, tmp);
+	if(shear) vmath_mul_matrix_matrix(rottransmat, shear, 4, 4, tmp);
 	vmath_mul_matrix_matrix(tmp, scalemat, 4, 4, out);
 }
 
