@@ -348,6 +348,20 @@ PyDoc_STRVAR(quat_rotation_doc,
 	"    quat = igeVmath.quat_rotation(float)\n"\
 	"        construct a quaternion to rotate around a Z(0,0,1) axis");
 
+//quat_look_rotation_doc
+PyDoc_STRVAR(quat_look_rotation_doc,
+	"Creates a rotation with the specified forward and upwards directions.\n"\
+	" - Z axis will be aligned with forward\n"\
+	" - X axis aligned with cross product between forwardand upwards\n"\
+	" - Y axis aligned with cross product between Z and X\n"\
+	"\n"\
+	"vmath.quat_look_rotation(forward: vmath.vec3, up: vmath.vec3)\n"
+	"\n"\
+	"Parameters\n"\
+	"----------\n"\
+	"    forward: forward vector\n"\
+	"    up: up vector\n");
+
 //quat_rotationX
 PyDoc_STRVAR(quat_rotationX_doc,
 	"construct a quaternion to rotate around the x axis\n"\
@@ -497,12 +511,12 @@ PyDoc_STRVAR(transpose_doc,
 
 //inverse
 PyDoc_STRVAR(inverse_doc,
-	"compute the inverse of a matrix and quaternion\n"\
+	"compute the inverse of a matrix or quaternion\n"\
 	"\n"\
 	"Forms\n"\
 	"----------\n"\
-	"	matrix = igeVmath.inverse(matrix)\n"\
-	"	quat = igeVmath.inverse(quat)");
+	"    matrix = igeVmath.inverse(matrix)\n"\
+	"    quat = igeVmath.inverse(quat)");
 
 
 //orthoInverse
